<?php
/* $Id: s_list.inc.php 4070 2010-10-19 09:30:29Z evgen $ */

/*
 * Generate objects list
 *
 * @param int objects Subdivision_ID
 * @param int objects Sub_Class_ID
 * @param string query string with params
 *  recNum - количество объектов
 *  list_mode - если 'select', показать кнопку "выбрать"
 *  ...
 *  for example: &isTitle=1&recNum=5
 * @param bool show result in admin mode
 *
 * @return HTML text
 */
function nc_objects_list ($sub, $cc, $query_string = "", $show_in_admin_mode = false) {
  
  $LIST_VARS = array();
  
  // parse query string in array
  parse_str($query_string, $LIST_VARS);
  
  // ignore this variables in query string
  $ignore_array = array(
    "nc_core", "inside_admin", "admin_mode", "catalogue", "cc", "cond_catalogue", "cond_catalogue_add", "cond_catalogue_addtable",
    "cond_cc", "cond_check", "cond_date", "cond_distinct",  "cond_group", "cond_having",  "cond_mod", "cond_parent",
    "cond_search", "cond_select", "cond_sub", "cond_user", "cond_where", "distinct", "distinctrow", "ignore_all", "ignore_limit", "ignore_cc", "ignore_check",
    "ignore_parent", "ignore_sub", "ignore_user", "ignore_calc", "ignore_prefix", "ignore_suffix", "query_from", "query_group", "query_join", "query_order", "query_select",
    "query_where", "query_group", "query_having", "query_limit", "result_vars", "sort_by", "sub", "user_table_mode",
    "cc_env", "field_vars", "message_select", "ignore_eval"
  );
  
  // flip array
  $ignore_array = array_flip($ignore_array);
  // ignore ignored variables
  $LIST_VARS = array_diff_key($LIST_VARS, $ignore_array);
  // extract
  extract($LIST_VARS);

  global $UI_CONFIG, $perm, $_cache, $admin_url_prefix, $classPreview;
  global $AUTH_USER_ID, $AUTH_USER_GROUP, $current_user;
  global $sub_level_count, $parent_sub_tree;
  // for old modules (forum)
  global $current_catalogue, $current_sub, $current_cc;

  // system superior object
  $nc_core = nc_Core::get_object();
  
  // system db object
  if ( is_object($nc_core->db) ) $db = &$nc_core->db;
  
  // modules variables
  $MODULE_VARS = $nc_core->modules->get_module_vars();
  // system variables
  $FILES_FOLDER = $nc_core->get_variable("FILES_FOLDER");
  $HTTP_ROOT_PATH = $nc_core->get_variable("HTTP_ROOT_PATH");
  $ADMIN_PATH = $nc_core->get_variable("ADMIN_PATH");
  $ADMIN_TEMPLATE = $nc_core->get_variable("ADMIN_TEMPLATE");
  $DOMAIN_NAME = $nc_core->get_variable("DOMAIN_NAME");
  $SHOW_MYSQL_ERRORS = $nc_core->get_variable("SHOW_MYSQL_ERRORS");
  $AUTHORIZE_BY = $nc_core->get_variable("AUTHORIZE_BY");
  $HTTP_FILES_PATH = $nc_core->get_variable("HTTP_FILES_PATH");
  $DOCUMENT_ROOT = $nc_core->get_variable("DOCUMENT_ROOT");
  $SUB_FOLDER = $nc_core->get_variable("SUB_FOLDER");
  // flags
  $inside_admin = $nc_core->inside_admin;
  $admin_mode = $nc_core->admin_mode;
  // current arrays
  $system_env = $nc_core->get_settings();
  $current_catalogue = $nc_core->catalogue->get_current();
  $current_sub = $nc_core->subdivision->get_current();
  $current_cc = $nc_core->sub_class->get_current();
  
  // ignore eval from $sort_by, $maxRows etc.
  // array is defined in the component settings
  // example: $ignore_eval['sort_by'] = true;
  $ignore_eval = array();

  
  // validate
 $catalogue = isset($catalogue) ? intval($catalogue) : 0;
 $sub = isset($sub) ? intval($sub) : 0;
 $cc = isset($cc) ? intval($cc) : 0;
 $template = isset($template) ? intval($template) : 0;
 $parent_message = isset($parent_message) ? intval($parent_message) : 0;
 $nc_ctpl = isset($nc_ctpl) ? intval($nc_ctpl) : 0;


  if (!$sub || !$cc) return false;
  if ( ($cc != $current_cc['Sub_Class_ID'] || !$isMainContent) && $admin_mode && !$show_in_admin_mode ) return false;
  
  try {
    $cc_env = $nc_core->sub_class->get_by_id($cc, null, $nc_ctpl);
  }
  catch ( Exception $e ) {
    return $e->getMessage();
  }

  if ( $cc_env['Type'] == 'rss' || $cc_env['Type'] == 'xml' ) {
    $cc_env['Cache_Access_ID'] = 2;
  }

  // если preview для нашего класса, то подменим cc_env из $_SESSION
  if ($classPreview == ($cc_env["Class_Template_ID"] ? $cc_env["Class_Template_ID"] : $cc_env["Class_ID"])) {
    $magic_gpc = get_magic_quotes_gpc();
    if ( !empty($_SESSION["PreviewClass"][$classPreview])) {
      foreach ($_SESSION["PreviewClass"][$classPreview] as $tkey => $tvalue) {
        $cc_env[$tkey] = $magic_gpc ? stripslashes($tvalue) : $tvalue;
      }
    }
    // Запретим кеширование в режиме предпросмотра.
    $cc_env['Cache_Access_ID'] = 2;
  }

  // Если присутствует параметр isSubClassArray в вызове функции s_list_class(), то добавляем
  // в массив $cc_env элемент cur_cc, который будет участвовать в формировании навигации по страницам
  // при отображении нескольких шаблонов на странице
  if ( isset($isSubClassArray) && $isSubClassArray)  $cc_env['cur_cc'] = $cc;

  if ($cc_env['Read_Access_ID'] > 1 && !$AUTH_USER_ID) return false;
  if ( $AUTH_USER_ID && $cc_env['Read_Access_ID'] > 2) {
    $HasRights = CheckUserRights($cc, 'read', 1);
    if (!$HasRights) return false;
  }

  // set user table mode
  $user_table_mode = (bool)$cc_env['System_Table_ID'];
  
  // cache section
  if ( nc_module_check_by_keyword("cache") && $cc_env['Cache_Access_ID']==1 && !$user_table_mode) {
    // startup values
    $cached_data = "";
    $cached_eval = false;

    try {
      $nc_cache_list = nc_cache_list::getObject();
      // cache auth addon string
      $cache_for_user = $nc_cache_list->authAddonString( $cc_env['CacheForUser'], $current_user);
      $cache_key = $query_string.$cache_for_user."type=".$cc_env['Type']."classtemplate=".$cc_env['ClassTemplate'];
      // check cached data
      $cached_result = $nc_cache_list->read($sub, $cc, $cache_key, $cc_env['Cache_Lifetime']);
      if ($cached_result!=-1) {
        // get cached parameters
        list ($cached_data, $cached_eval, $cache_vars) = $cached_result;
        // debug info
        $cache_debug_info = "Readed, sub[".$sub."], cc[".$cc."], Access_ID[".$cc_env['Cache_Access_ID']."], Lifetime[".$cc_env['Cache_Lifetime']."], bytes[".strlen($cached_data)."], eval[".(int)$cached_eval."]";
        $nc_cache_list->debugMessage($cache_debug_info, __FILE__, __LINE__);
        // return cache if not eval flag setted
        if (!$cached_eval) {
          $result = $cached_data;
          return $result;
        }
      }
      // set marks into the fields
      $no_cache_marks = $nc_cache_list->nocacheStore($cc_env);
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_list->errorMessage($e);
    }
  }

  $allowTags = $cc_env['AllowTags'];
  $NL2BR = $cc_env['NL2BR'];
  $catalogue = $cc_env['Catalogue_ID'];
  $intQueryStr = '?';

  $cc_settings = &$cc_env["Sub_Class_Settings"];
  
  // current host
  $subHost = "http://".($cc_env['Hidden_Host'] ? ( strchr($cc_env['Hidden_Host'], ".") ? $cc_env['Hidden_Host'] : $cc_env['Hidden_Host'].".".$DOMAIN_NAME ) : $DOMAIN_NAME);
  
  // $subLink, $ccLink, $cc_keyword
  if ($admin_mode) {
    $subLink = $admin_url_prefix.'?catalogue='.$catalogue.'&amp;sub='.$sub;
    $ccLink = $subLink.'&amp;cc='.$cc;
    $intQueryStr = $ccLink;
  }
  else {
    $subLink = $SUB_FOLDER.$cc_env['Hidden_URL'];
    $cc_keyword = $cc_env['EnglishName'];
    $ccLink = $subLink.$cc_keyword.'.html';
  }

  // переменные curPos, recNum нужно привести к "правильному" виду
  // до И после выполнения системных настроек компонента
  $curPos = isset($curPos) ? intval($curPos) : 0;
  if ($curPos < 0) $curPos = 0;
  $recNum = isset($recNum) ? intval($recNum) : 0;
  if ($recNum < 0) $recNum = 0;

  // if RecordsPerPage not setted in component - set ignore_limit
  $ignore_limit = ( !($maxRows = $cc_env['RecordsPerPage']) );
  $sortBy = $cc_env['SortBy'];
  $classID = $cc_env['Class_ID'];
  $userTableID = $cc_env['System_Table_ID'];

  $ignore_all = 0;
  $ignore_catalogue = $ignore_sub = $ignore_cc = 0;
  $ignore_check = $ignore_parent = 0;
  $ignore_prefix = $ignore_suffix = 0;
  $ignore_calc = 0;
  $ignore_user = true;
  $result_vars = '';
  $cond_date = '';
  $no_cache_marks = 0;

  if ( $MODULE_VARS['search']['INDEX_TABLE'] == $classID ) $ignore_eval['sort_by'] = true;

  // component "system settings"
  if ($cc_env['Settings']) {
    eval ($cc_env['Settings']);
  }

  // переменные попадают в запрос
  // нужно привести к целому типу
  $curPos = intval($curPos);
  if ($curPos < 0) $curPos = 0;
  $recNum = intval($recNum);
  if ($recNum < 0) $recNum = 0;
  
  // clear this variable after system settings eval!
  $result = "";
  
  if (!$sortBy) {
    $sort_by = "a.".($user_table_mode ? "`".$AUTHORIZE_BY."`" : "`Priority` DESC").", a.`LastUpdated` DESC";
  }
  else {
    $sort_by = $sortBy;
  }

  //выйдем, если нет идентификатора шаблона, поскольку дальше работа функции бессмысленна
  if (!$classID) return false;

  $component = new nc_Component( $classID, $cc_env['System_Table_ID']);
  $component->make_query();

  $field_names = $component->get_fields_query();
  $field_vars = $component->get_fields_vars();
  $multilist_fileds = $component->get_fields(10);
  $date_field = $component->get_date_field();

  // разрешить html-теги и перенос строки
  $cc_env['convert2txt'] = "";
  $text_fields = $component->get_fields(3);
  foreach ( $text_fields as $field ) {
    $format = nc_field_parse_format($field['format'], 3);
    // разрешить html
    if (!$cc_env['AllowTags'] && !$format['html'] || $format['html'] == 2 ) $cc_env['convert2txt'] .= "\$f_".$field['name']." = htmlspecialchars(\$f_".$field['name'].");";
    // перенос строки
    if ($cc_env['NL2BR'] && !$format['br'] || $format['br'] == 1) $cc_env['convert2txt'] .= "\$f_".$field['name']." = nl2br(\$f_".$field['name'].");";
    if ($format['bbcode'] ) $cc_env['convert2txt'] .= "\$f_".$field['name']." = nc_bbcode(\$f_".$field['name'].",  (\$fullDateLink ? \$fullDateLink : \$fullLink) );";
  }
  $text_fields = $component->get_fields(1);
  foreach ( $text_fields as $field ) {
    if (!$cc_env['AllowTags']  ) $cc_env['convert2txt'] .= "\$f_".$field['name']." = htmlspecialchars(\$f_".$field['name'].");";
    if ($cc_env['NL2BR'] ) $cc_env['convert2txt'] .= "\$f_".$field['name']." = nl2br(\$f_".$field['name'].");";
  }
  unset( $format);
  unset($text_fields);


 // $fullSearchParams = getSearchParams($field_name, $field_type, $field_search, $srchPat);

  $fullSearchParams = $component->get_search_query( isset($srchPat) ? $srchPat : array()) ;

  $fullSearchStr = $fullSearchParams['query'];
  $fullSearchURL = $fullSearchParams['link'];

  if (!$ignore_catalogue) {
    $cond_catalogue = " AND sub.`Catalogue_ID` = '".$catalogue."' ";
    $cond_catalogue_add = " AND a.`Subdivision_ID` = sub.`Subdivision_ID` ";
    $cond_catalogue_addtable = ", `Subdivision` AS sub ";
  }

  $cond_sub  = !$ignore_sub ? " AND a.`Subdivision_ID` = '".$sub."' " : "";
  $cond_cc   = !$ignore_cc ?  " AND a.`Sub_Class_ID` = '".$cc."' " : "";
  $cond_user = !$ignore_user ? " AND a.`User_ID` = '".$AUTH_USER_ID."' " : "";
  $cond_parent = !$ignore_parent ? " AND a.`Parent_Message_ID` = '".$parent_message."' " : "";
  $cond_search = $fullSearchStr;
  if (!$admin_mode && !$ignore_check) $cond_mod = " AND a.`Checked` = 1 ";
  if ( isset($date) && $date && $date_field && strtotime($date) > 0) $cond_date = " AND a.`".$date_field."` LIKE '".$db->escape($date)."%' ";

  $cond_distinct = isset($distinct) && $distinct ? "DISTINCTROW" : "";
  $cond_distinct = isset($distinctrow) && $distinctrow ? "DISTINCTROW" : "";
  $cond_select = isset($query_select) && $query_select ? ", ".$query_select : "";
  $cond_where = isset($query_where) && $query_where ? " AND ".$query_where : "";
  $cond_group = isset($query_group) && $query_group ? " GROUP BY ".$query_group : "";
  $cond_having = isset($query_having) && $query_having ? " HAVING ".$query_having : "";
  if ( isset($query_order) && $query_order) $sort_by = $query_order;
  if ( !isset($query_from) ) $query_from = '';
  if ( !isset($query_join) ) $query_join = '';



 
  
  if ($user_table_mode) {
    $cond_sub = "";
    $cond_cc = "";
    $cond_catalogue_add = "";
    $cond_catalogue = "";
    $cond_catalogue_addtable = "";
    $cond_parent = "";
  }

  if ( $fullSearchURL ) $intQueryStr .= (($intQueryStr == '?') ?  '' : '&amp;') .$fullSearchURL;

  if (!$recNum) {
    $recNum = $maxRows;
  }
  else {
    $maxRows = $recNum;
    // для совместимости со старыми версиями до 2.4.5 и 3.0.0
    if (!isset($ignore_eval['maxRows']) || !$ignore_eval['maxRows']) eval("\$maxRows = \"".$maxRows."\";");
  }

  $maxRows = intval($maxRows);



  // для совместимости со старыми версиями до 2.4.5 и 3.0.0
  if ( !isset($ignore_eval['sort_by']) ||  !$ignore_eval['sort_by']) eval("\$sort_by = \"".$sort_by."\";");


  if (!$ignore_all) {
    $message_select = "SELECT".(!$ignore_calc ? " SQL_CALC_FOUND_ROWS" : "")." ".$cond_distinct." ".$field_names.$cond_select." 
                       FROM (".($user_table_mode ? "`User`" : "`Message".$classID."`")." AS a ".( $query_from ? ", ".$query_from : "").")
                       ".$component->get_joins()." ".$query_join." 
                       WHERE 1 ".$cond_parent.$cond_where.$cond_catalogue.$cond_sub.$cond_cc.$cond_user.$cond_mod.$cond_search.$cond_date.
                       $cond_group.
                       $cond_having.
                       ($sort_by ? " ORDER BY ".$sort_by : "").
                       (!$ignore_limit ? " LIMIT ".( isset($cc_env['cur_cc']) && $cc_env['cur_cc']==$cur_cc ? $curPos : ( !isset($cc_env['cur_cc']) ? $curPos : "0") ).",".$maxRows : ($query_limit ? " LIMIT ".$query_limit : "") );
  }
  elseif ($query_select && $query_from) {
    $message_select = "SELECT".(!$ignore_calc ? " SQL_CALC_FOUND_ROWS" : "")." ".$query_select." FROM ".$query_from.
    ($query_join   ?  " ".$query_join : "").
    ($query_where  ?  " WHERE ".$query_where : "").
    ($query_group  ?  " GROUP BY ".$query_group : "").
    ($query_having ?  " HAVING  ".$query_having : "").
    ($query_order  ?  " ORDER BY ".$query_order : "").
    ($query_limit  ?  " LIMIT ".$query_limit : "");
  }

  $addLink = $SUB_FOLDER.$cc_env['Hidden_URL']."add_".$cc_env['EnglishName'].".html";
  $rssLink = $cc_env['AllowRSS'] ?  $SUB_FOLDER.$cc_env['Hidden_URL'].$cc_env['EnglishName'].".rss" : "";
  $xmlLink = $cc_env['AllowXML'] ?  $SUB_FOLDER.$cc_env['Hidden_URL'].$cc_env['EnglishName'].".xml" : "";
  $subscribeLink = $SUB_FOLDER.$cc_env['Hidden_URL']."subscribe_".$cc_env['EnglishName'].".html";
  $searchLink = $SUB_FOLDER.$cc_env['Hidden_URL']."search_".$cc_env['EnglishName'].".html";

  //$nc_search_form = "<form method='get' action='".$SUB_FOLDER.$cc_env['Hidden_URL']."'>".showSearchForm($field_descript, $field_type, $field_search, $field_format)."<input type='submit' value='".NETCAT_SEARCH_FIND_IT."'/></form>";

  $cc_env['LocalQuery'] = $intQueryStr;
  $cc_env['curPos'] = $curPos;
  $cc_env['dateField'] = $date_field;
  $cc_env['recNum'] = $recNum;
  $cc_env['maxRows'] = $maxRows;
  $cc_env['addLink'] = $addLink;
  $cc_env['subscribeLink'] = $subscribeLink;
  $cc_env['searchLink'] = $searchLink;
  $cc_env['fieldCount'] = count($component->get_fields());
  //$cc_env['nc_search_form'] = $nc_search_form;

  // cache eval section
  if ( nc_module_check_by_keyword("cache") && $cc_env['Cache_Access_ID']==1 && is_object($nc_cache_list) && $cached_eval && $cached_result!=-1) {
    // get cached objects blocks
    $component_cache_blocks = $nc_cache_list->getCachedBlocks($cached_data);

    // cached prefix
    eval ("\$result = \"".$component_cache_blocks['prefix']."\";");
    
    if ( is_array($component_cache_blocks) && !empty($component_cache_blocks) ) {
      // concat cached objects
      foreach ($component_cache_blocks['objects'] as $k => $v) {
        // extract cached object variables
        if ( !empty($cache_vars) && is_array($cache_vars[$k]) ) {
          extract($cache_vars[$k]);
        }
        // append obect data
        eval ("\$result.= \"".$v."\";");
      }
    }
    
    // cached suffix
    eval ("\$result.= \"".$component_cache_blocks['suffix']."\";");

    return $result;
  }
  
  // in this section check usable or not add/search forms in component
  $cc_env["AddTemplate"] = $cc_env["AddTemplate"] ? $cc_env["AddTemplate"] : $component->add_form($catalogue, $sub, $cc);
  $cc_env["FullSearchTemplate"] = $cc_env["FullSearchTemplate"] ? $cc_env["FullSearchTemplate"] :  $component->search_form(1);
  $component_body = (!$ignore_prefix ? $cc_env['FormPrefix'] : "").(!$ignore_suffix ? $cc_env['FormSuffix'] : "").$cc_env['RecordTemplate'].$cc_env['RecordTemplateFull'].$cc_env['Settings'];
  if ( nc_strpos($component_body, '$addForm')    !== false ) eval("\$addForm = \"".$cc_env["AddTemplate"]."\";");
  if ( nc_strpos($component_body, '$searchForm') !== false ) eval("\$searchForm = \"".$cc_env["FullSearchTemplate"]."\";");
  unset ($component_body);
    // ==================================================

  
  $db->last_error = "";
  
  // main query execution
  if ($message_select) {
    $res = $db->get_results($message_select, ARRAY_N);
  }


  // error in query
  if ($db->last_error) {
    // determine error
    switch (true) {
      case preg_match("/Table '\w+\.Classificator_(\w+)' doesn't exist/i", $db->last_error, $regs):
        $err = sprintf(NETCAT_FUNCTION_OBJECTS_LIST_CLASSIFICATOR_ERROR, $regs[1]);
      break;
      case preg_match("/Unknown column '(.+?)' in 'field list'/i", $db->last_error, $regs):
        $err = sprintf(NETCAT_FUNCTION_OBJECTS_LIST_SQL_COLUMN_ERROR_UNKNOWN, $regs[1]);
      break;
      case preg_match("/Unknown column '(.+?)' in 'order clause'/i", $db->last_error, $regs):
        $err = sprintf(NETCAT_FUNCTION_OBJECTS_LIST_SQL_COLUMN_ERROR_CLAUSE, $regs[1]);
      break;
      case $SHOW_MYSQL_ERRORS=="on":
        $err = $db->last_error;
      break;
      default:
        $err = "";
    }
    
    // error message
    if ( is_object($perm) &&  $perm->isSupervisor() ) {
      // error info for the supervisor
      trigger_error( sprintf( NETCAT_FUNCTION_OBJECTS_LIST_SQL_ERROR_SUPERVISOR, $sub, $cc, $query_string, ($err ? $err.", " : "") ), E_USER_WARNING );
    }
    else {
      // error info for the simple users
      echo NETCAT_FUNCTION_OBJECTS_LIST_SQL_ERROR_USER;
    }
    // return
    return false;
  }

  if ($message_select) {
    // object in this page
    $rowCount = $db->num_rows;
    // total objects
    $totRows = !$ignore_calc ? $db->get_var("SELECT FOUND_ROWS()") : $rowCount;
    $totRows += 0;
  }
  else {
    $rowCount = 0;
    $totRows = 0;
  }
  
 	$_get_arr = $nc_core->input->fetch_get();
  $get_param_str = '';
	// в nextLink и prevLink нужно сохранить get-параметры
  if ( !empty($_get_arr) ) {
    // ignore array
    $ignore_arr = array('sid', 'ced', 'inside_admin', 'catalogue', 'sub', 'cc', 'curPos', 'cur_cc', 'REQUEST_URI');
	  // check
    foreach ($_get_arr as $k => $v) {
	    if ( !in_array($k, $ignore_arr) ) $get_param[$k] = $v;
	  }
    // build string
    if ( !empty($get_param) ) $get_param_str = $nc_core->url->build_url($get_param); 
	}
  // clear
  unset ($_get_arr);
	
  $begRow = $curPos + 1;
  $prevLink = $nextLink = '';
  if ($curPos >= $maxRows) {
    $prevLink = $cc_env['LocalQuery'].(nc_strlen($cc_env['LocalQuery']) > 1 ? "&amp;" : "")."curPos=".($curPos-$maxRows).($cc_env['cur_cc'] ? "&amp;cur_cc=".$cc_env['cur_cc'] : "").($classPreview == $cc_env["Class_ID"] ? "&amp;classPreview=".$classPreview : "").($get_param_str ? "&amp;".$get_param_str : "");
  }

  $endRow = $curPos + $maxRows;
  if ($endRow < $totRows) {
    $nextLink = $cc_env['LocalQuery'].(nc_strlen($cc_env['LocalQuery']) > 1 ? "&amp;" : "")."curPos=".$endRow.( isset($cc_env['cur_cc']) && $cc_env['cur_cc'] ? "&amp;cur_cc=".$cc_env['cur_cc'] : "").($classPreview == $cc_env["Class_ID"] ? "&amp;classPreview=".$classPreview : "").($get_param_str ? "&amp;".$get_param_str : "");
  }
  else {
    $endRow = $totRows;
  }

  $cc_env['begRow'] = $begRow;
  $cc_env['endRow'] = $endRow;
  $cc_env['totRows'] = $totRows;
  $cc_env['prevLink'] = $prevLink;
  $cc_env['nextLink'] = $nextLink;

  if (!$ignore_all) {
    $fetch_row = "list(".$field_vars.($result_vars ? ", ".$result_vars : "").") = \$res[\$f_RowNum];";
  }
  else {
    $fetch_row = $result_vars ? "list(".$result_vars.") = \$res[\$f_RowNum];" : "";
  }
 
  // Право на модерирование и изменение объектов.
  $modPerm = false;
  $changePerm = false;

  if ($admin_mode) {
    $modPerm = CheckUserRights($cc, 'moderate', 1); // право модератора
    $changePerm = s_auth ($cc_env, 'change', 1); //               или просто на изменение объектов

    if ( is_object($perm) && $perm->isBanned($cc_env, 'change')   ) $modPerm = $changePerm = false; // пользователю запретили изменение объектов
    
    //Номер шаблона в разделе
    $f_AdminCommon_cc = $cc;
    //Название шаблона в разделе
    $f_AdminCommon_cc_name = $cc_env['Sub_Class_Name'];
    // добавить
    $f_AdminCommon_add = $admin_url_prefix."add.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc;
    $addLink = $f_AdminCommon_add;
    // удалить все
    $f_AdminCommon_delete_all = $admin_url_prefix."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;delete=1";
    // экспорт и импорт в CSV
    $f_AdminCommon_export_csv = $admin_url_prefix."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;export=1";
    $f_AdminCommon_import_csv = $admin_url_prefix."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;import=1";
    // экспорт и импорт в XML
    $f_AdminCommon_export_xml = $admin_url_prefix."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;export=2";
    $f_AdminCommon_import_xml = $admin_url_prefix."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;import=2";
    
    // Js и форма для пакетной обработки объектов
    $f_AdminCommon_package = "<script type='text/javascript' language='javascript'>\n";
    $f_AdminCommon_package.= "\tnc_package_obj.new_cc(".$cc.", '".NETCAT_MODERATION_NOTSELECTEDOBJ."');\n";
    $f_AdminCommon_package.= "</script>\n";
    $f_AdminCommon_package.= "<form id='nc_form_selected_".$cc."' action='".$SUB_FOLDER.$HTTP_ROOT_PATH."message.php' method='post'>\n";
    $f_AdminCommon_package.= "\t<input type='hidden' name='catalogue' value='".$catalogue."'>\n";
    $f_AdminCommon_package.= "\t<input type='hidden' name='sub' value='".$sub."'>\n";
    $f_AdminCommon_package.= "\t<input type='hidden' name='cc' value='".$cc."'>\n";
    $f_AdminCommon_package.= "\t<input type='hidden' name='curPos' value='".$curPos."'>\n";
    $f_AdminCommon_package.= "\t<input type='hidden' name='admin_mode' value='".$admin_mode."'>\n";
    $f_AdminCommon_package.= "</form>\n";

    if ($list_mode!="select") {
    
      if ($inside_admin && $isMainContent && $UI_CONFIG) {
        // в админке нет AdminCommon, но нужна часть для пакетной обработки
        if ($totRows!=0) $result.= $f_AdminCommon_package;
        // add button
        $UI_CONFIG->actionButtons = array();
        $UI_CONFIG->actionButtons[] = array(
          "id" => "addObject",
          "caption" => NETCAT_MODERATION_BUTTON_ADD,
          "action" => "urlDispatcher.load('object.add(".$cc.")')"
        );
        // edit button
        $UI_CONFIG->actionButtons[] = array(
          "id" => "windowEdit",
          "caption" => SUBDIVISION_TAB_EDIT_BUTTON_PREVIEW,
          "action" => "urlDispatcher.load('".$admin_url_prefix."?catalogue=".$catalogue."&sub=".$sub."&cc=".$cc."', '1')"
        );
        // кнопки пакетной обработки нужны только если есть объекты
        if ($totRows!=0) {
          //  button "delete all"
          $UI_CONFIG->actionButtons[] = array(
            "id" => "deleteAll",
            "caption" => NETCAT_MODERATION_REMALL,
            "align" => "left",
            "action" => "urlDispatcher.load('subclass.purge(".$cc.")')"
          );
          // button "Удалить выбранные"
          $UI_CONFIG->actionButtons[] = array(
            "id" => "deleteChecked",
            "caption" => NETCAT_MODERATION_DELETESELECTED,
            "align" => "left",
            "action" => "document.getElementById('mainViewIframe').contentWindow.nc_package_obj.process('delete', ".$cc.")"
          );
          // button "Выключить выбранные"
          $UI_CONFIG->actionButtons[] = array(
            "id" => "checkOff",
            "caption" => NETCAT_MODERATION_SELECTEDOFF,
            "align" => "left",
            "action" => "document.getElementById('mainViewIframe').contentWindow.nc_package_obj.process('checkOff', ".$cc.")"
          );
          // button "Включить выбранные"
          $UI_CONFIG->actionButtons[] = array(
            "id" => "checkOn",
            "caption" => NETCAT_MODERATION_SELECTEDON,
            "align" => "left",
            "action" => "document.getElementById('mainViewIframe').contentWindow.nc_package_obj.process('checkOn', ".$cc.")"
          );
        }
      }

      if (!$inside_admin) {
        if ($system_env['AdminButtonsType']) {
          eval("\$f_AdminCommon = \"".$system_env['AdminCommon']."\";");
        }
        else {
          $f_AdminCommon_buttons = "<a href='#' onclick='nc_package_obj.process(\"checkOn\", ".$cc."); return false;'><img src='".$ADMIN_TEMPLATE."img/i_selected_on.gif' alt='".NETCAT_MODERATION_SELECTEDON."' title='".NETCAT_MODERATION_SELECTEDON."'></a>";
          $f_AdminCommon_buttons .= "<a href='#' onclick='nc_package_obj.process(\"checkOff\", ".$cc."); return false;'><img src='".$ADMIN_TEMPLATE."img/i_selected_off.gif' alt='".NETCAT_MODERATION_SELECTEDOFF."' title='".NETCAT_MODERATION_SELECTEDOFF."'></a>";
          $f_AdminCommon_buttons .= "&nbsp;&nbsp;";
          $f_AdminCommon_buttons .= "<a href='#' onclick='nc_package_obj.process(\"delete\", ".$cc."); return false;'><img src='".$ADMIN_TEMPLATE."img/i_selected_remove.gif' alt='".NETCAT_MODERATION_DELETESELECTED."' title='".NETCAT_MODERATION_DELETESELECTED."'></a>";
          $f_AdminCommon_buttons .= "&nbsp;&nbsp;";
          $f_AdminCommon_buttons .= "<a href='".$f_AdminCommon_add."'><img src='".$ADMIN_TEMPLATE."img/i_obj_add.gif' alt='".CONTROL_CONTENT_CATALOUGE_FUNCS_SHOWMENU_ADD."' title='".CONTROL_CONTENT_CATALOUGE_FUNCS_SHOWMENU_ADD."'></a>";
          $f_AdminCommon_buttons.= "<a href='".$f_AdminCommon_delete_all."'><img src='".$ADMIN_TEMPLATE."img/i_obj_delete_all.gif' alt='".NETCAT_MODERATION_REMALL."' title='".NETCAT_MODERATION_REMALL."'></a>";

          $f_AdminCommon = "<div class='nc_idtab nc_admincommon'>\r\n";
          if ( CheckUserRights( $cc, 'add', 1 )==1 ) {
            $f_AdminCommon.= "<div class='nc_idtab_id'>\r\n";
            $f_AdminCommon.= "<div class='nc_idtab_ccid' title='".NETCAT_MODERATION_CLASSID.": ".$f_AdminCommon_cc."'>".$f_AdminCommon_cc.".</div>\r\n";
            $f_AdminCommon.= "<div class='nc_idtab_ccname'>".$f_AdminCommon_cc_name."</div>\r\n";
            $f_AdminCommon.= "</div>\r\n";
            $f_AdminCommon.= "<div class='nc_idtab_buttons'>".$f_AdminCommon_buttons."</div>\r\n";
            $f_AdminCommon.= "<div class='ncf_row nc_clear'></div>\r\n";
            $f_AdminCommon .= $f_AdminCommon_package;
          }
          else {
            $f_AdminCommon.= "<div class='nc_idtab_id'>\r\n";
            $f_AdminCommon.= "<div class='nc_idtab_messageid error' title='".NETCAT_MODERATION_ERROR_NORIGHT."'>".NETCAT_MODERATION_ERROR_NORIGHT;
            $f_AdminCommon.= "</div>\r\n";
            $f_AdminCommon.= "</div>\r\n";
            $f_AdminCommon.= "<div class='ncf_row nc_clear'></div>\r\n";
          }
          $f_AdminCommon.= "</div>\r\n";
          $f_AdminCommon.= "<div class='ncf_row nc_clear'></div>\r\n";
          
        
        }
      }

    }

  }
  else {
    // not admin mode
    $f_AdminCommon_cc = "";
    $f_AdminCommon_cc_name = "";
    $f_AdminCommon_add = "";
    $f_AdminCommon_delete_all = "";
    $f_AdminCommon = "";
    $f_AdminButtons = "";
  }

  $row_ids = array(); // массив, в который будут складываться ID всех узлов
  for ( $i = 0; $i < $rowCount; $i++ ) {
    $row_ids[] = $res[$i][0];
  }

  // component prefix
  if ($cc_env['FormPrefix'] && !$ignore_prefix) {
    eval("\$result.= \"".$cc_env["FormPrefix"]."\";");
  }
  else {
    $result.= $f_AdminCommon;
  }

  // если список пуст, внутри админки нужно показать сообщение "нет объектов"
  if ($inside_admin && $totRows==0 && !nc_strlen( trim($result) )) {
    $result.= "<div class='status_info'>".NETCAT_MODERATION_NO_OBJECTS_IN_SUBCLASS."</div>";
  }
  
 
  
  // получаем все файлы выбранных объектов
  $nc_fields_files = $component->get_fields(6, 0);
  $nc_files_in_class = array();

  if ( !empty($nc_fields_files) &&  !empty($row_ids) ) {
    // get data for all file_fields
    $filetable = $db->get_results("SELECT `Field_ID`, `Message_ID`, `Virt_Name`, `Real_Name`, `Download`
                                   FROM `Filetable`
                                   WHERE `Field_ID` IN (".join( ", ", array_keys($nc_fields_files) ).")
                                   AND `Message_ID` IN (".join(", ", $row_ids).")" , ARRAY_A);

    // sorting array
    if ( !empty($filetable) ) foreach ($filetable AS $v) {
      $nc_files_in_class[ $v['Message_ID'] ][ $v['Field_ID'] ] = array($v['Virt_Name'], $v['Real_Name'], $v['Download']);
    }
    unset ($filetable);
  }
  
  // требуется получить все группы пользователей
  if ( $user_table_mode && !empty($row_ids) ) {
    $nc_user_group = $db->get_results("SELECT ug.`User_ID`, ug.`PermissionGroup_ID`, g.`PermissionGroup_Name` 
                                       FROM `User_Group` AS ug,`PermissionGroup` AS g
                                       WHERE User_ID IN (".join(', ', $row_ids).")
                                       AND g.`PermissionGroup_ID` = ug.`PermissionGroup_ID` ", ARRAY_A);
    if ( !empty($nc_user_group ) ) {
      foreach ( $nc_user_group as $v ) {
        $nc_user_group_sort[$v['User_ID']][$v['PermissionGroup_ID']] = $v['PermissionGroup_Name']; 
      }
    }
    unset($nc_user_group);
  }
  

  // =========================  Листинг объектов ======================================
  
  $cache_vars = array();
  
  for($f_RowNum = 0; $f_RowNum < $rowCount; $f_RowNum++) {
    // all result vars
    eval($fetch_row);
    
    if ( $no_cache_marks) {
      // caching variables array
      $cache_vars[$f_RowNum] = array();
      // get variables names string
      preg_match("/^list\((.*?)\).*?$/", $fetch_row, $matches);
      if ($matches[1]) {
        // variables by name array
        $cache_vars_name = explode(",", $matches[1]);
        if ( !empty($cache_vars_name) ) {
          // correcting
          foreach ($cache_vars_name as $k => $v) {
            $_variable_name = trim( str_replace('$', "", $v ) );
            $cache_vars[$f_RowNum][$_variable_name] = $$_variable_name;
            // clear
            unset($_variable_name);
          }
          // clear
          unset($cache_vars_name);
        }
      }      
    }

    if ( !$user_table_mode && $admin_mode && $AUTHORIZE_BY==='User_ID' ){
      $f_AdminInterface_user_add = $f_UserID;
      $f_AdminInterface_user_change = $f_LastUserID;
    }

    
    // Multiselect
    if ( !empty($multilist_fileds) ) {
      // просмотр каждого поля типа multiselect
      foreach ( $multilist_fileds as $multilist_filed) {
        // таблицу с элементами можно взять их кэша, если ее там нет - то добавить
        if ( !$_cache['classificator'][$multilist_filed['table']] ) {
          $db_res = $db->get_results("SELECT `".$multilist_filed['table']."_ID` AS ID, `".$multilist_filed['table']."_Name` AS Name, `Value`
                                       FROM `Classificator_".$multilist_filed['table']."`", ARRAY_A);
          if ( !empty($db_res)) {
            foreach ( $db_res as $v) { // запись в кэш
              $_cache['classificator'][$multilist_filed['table']][$v['ID']] = array($v['Name'], $v['Value']);  
            }
          }
          unset($db_res);
        }
        

        ${"f_".$multilist_filed['name']."_id"} = array();
        ${"f_".$multilist_filed['name']."_value"} = array();
        
        if ( ($value = ${"f_".$multilist_filed['name']} ) ) { // значение из базы
          ${"f_".$multilist_filed['name']} = array();
          ${"f_".$multilist_filed['name']."_id"} = array();
          $ids = explode(',', $value);
          if ( !empty($ids) ) {
            foreach ( $ids as $id ) { // для каждого элемента по id определяем имя и значение
              if ($id) {
                array_push(${"f_".$multilist_filed['name']}, $_cache['classificator'][$multilist_filed['table']][$id][0]);
                array_push(${"f_".$multilist_filed['name']."_value"}, $_cache['classificator'][$multilist_filed['table']][$id][1]);
                array_push(${"f_".$multilist_filed['name']."_id"}, $id);
              }
            }
          }
        }
        // default values
        if ( !is_array(${"f_".$multilist_filed['name']}) ) ${"f_".$multilist_filed['name']}  = array();
   
      }
      unset($ids);
      unset($id);
      unset($value);
    } //end multiselect
 

    // get files for message
    if ( !empty($nc_fields_files) ) {
              
      foreach ($nc_fields_files AS $field_id => $field_name) {
        unset(${"f_".$field_name."_name"});
        unset(${"f_".$field_name."_type"});
        unset(${"f_".$field_name."_size"});
        unset(${"f_".$field_name."_download"});
        unset(${"f_".$field_name."_url"});
        //  apparently we don't have a file
        if (!${"f_".$field_name}) {
          ${"f_".$field_name."_url"} = "";
          continue;
        }
        //file_data - массив с ориг.названием, типом, размером, [именем_файла_на_диске]
        $file_data = explode(':', ${"f_".$field_name});
        
        
        $filetable_path = ($user_table_mode ? "u" : $f_Subdivision_ID."/".$f_Sub_Class_ID);
        ${"f_".$field_name."_name"} = $file_data[0];
        ${"f_".$field_name."_type"} = $file_data[1];
        ${"f_".$field_name."_size"} = $file_data[2];
        // обнулим 
        $Virt_Name = "";
        $nc_download = 0;
        
        // get data from files array
        if ( is_array($nc_files_in_class[$f_RowID][$field_id]) ) {
          list($Virt_Name, $Real_Name, $nc_download) = $nc_files_in_class[$f_RowID][$field_id];
        }
        
        if ($Virt_Name) { // файловая система c Filetable
          ${"f_".$field_name} = $SUB_FOLDER.$HTTP_FILES_PATH.$filetable_path."/h_".$Virt_Name;
          ${"f_".$field_name."_url"} = $SUB_FOLDER.$HTTP_FILES_PATH.$filetable_path."/".$Virt_Name;
          ${"f_".$field_name."_download"} = $nc_download;
        }
        else {
          if ( $file_data[3] ) { // файловая система "Original"
            ${"f_".$field_name} = ${"f_".$field_name."_url"} = $SUB_FOLDER.$HTTP_FILES_PATH.$file_data[3];       
          }
          else {
            $ext = substr($file_data[0], strrpos($file_data[0], "."));
            ${"f_".$field_name."_url"} = ${"f_".$field_name} = $SUB_FOLDER.$HTTP_FILES_PATH.$field_id."_".$f_RowID.$ext;          
          }
        }
      }
    }
    
    if ($user_table_mode) {
      $f_PermissionGroup = &$nc_user_group_sort[$f_RowID];
    }

    // created date
    $f_Created_year = substr($f_Created, 0, 4);
    $f_Created_month = substr($f_Created, 5, 2);
    $f_Created_day = substr($f_Created, 8, 2);
    $f_Created_hours = substr($f_Created, 11, 2);
    $f_Created_minutes = substr($f_Created, 14, 2);
    $f_Created_seconds = substr($f_Created, 17, 2);

    // last updated date
    $f_LastUpdated_year = substr($f_LastUpdated, 0, 4);
    $f_LastUpdated_month = substr($f_LastUpdated, 4, 2);
    $f_LastUpdated_day = substr($f_LastUpdated, 6, 2);
    $f_LastUpdated_hours = substr($f_LastUpdated, 8, 2);
    $f_LastUpdated_minutes = substr($f_LastUpdated, 10, 2);
    $f_LastUpdated_seconds = substr($f_LastUpdated, 12, 2);

    if ($admin_mode) {
      $dateLink = '';
      if ($date_field) eval("\$dateLink = \"&date=\".\$f_".$date_field."_year.\"-\".\$f_".$date_field."_month.\"-\".\$f_".$date_field."_day;");
      // full link for object
      $fullLink = $admin_url_prefix."full.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;message=".$f_RowID;
      $fullDateLink = $fullLink.$dateLink;
      // ID объекта в шаблоне
      $f_AdminButtons_id = $f_RowID;
      // Приоритет объекта
      $f_AdminButtons_priority = $f_Priority;
      // ID добавившего пользователя
      $f_AdminButtons_user_add = $f_UserID;
      // ID изменившего пользователя
      $f_AdminButtons_user_change = $f_LastUserID ? $f_LastUserID : "";
      // копировать объект
      $f_AdminButtons_copy = $ADMIN_PATH."objects/copy_message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;message=".$f_RowID;
      // изменить
      $f_AdminButtons_change = $SUB_FOLDER.$HTTP_ROOT_PATH."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;message=".$f_RowID.( $curPos ? "&amp;curPos=".$curPos : "");
      $editLink = $f_AdminButtons_change;
      // удалить
      $f_AdminButtons_delete = $SUB_FOLDER.$HTTP_ROOT_PATH."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;message=".$f_RowID."&amp;delete=1".( $curPos ? "&amp;curPos=".$curPos : "");
      $deleteLink = $f_AdminButtons_delete;
      $dropLink = $deleteLink."&posting=1".($nc_core->token->is_use('drop') ? "&".$nc_core->token->get_url() : "");
      // включить-выключить
      $f_AdminButtons_check = ($f_Checked
        ? $any_url_prefix.$SUB_FOLDER.$HTTP_ROOT_PATH."message.php?catalogue=".$catalogue."&amp;sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;message=".$f_RowID."&amp;checked=1&amp;posting=1".( $curPos ? "&amp;curPos=".$curPos : "").($admin_mode ? "&amp;admin_mode=1" : "")
        : $admin_url_prefix."message.php?sub=".$sub."&amp;cc=".$cc."&amp;classID=".$classID."&amp;message=".$f_RowID."&amp;checked=2&amp;posting=1".( $curPos ? "&amp;curPos=".$curPos : "").($admin_mode ? "&amp;admin_mode=1" : "")
      );
      $checkedLink = $f_AdminButtons_check;
      // выбрать связанный (JS код!!!) -- когда список вызван в popup для выбора связанного объекта
      $f_AdminButtons_select = "top.selectItem(".$f_AdminButtons_id."); return false;";
      
      $f_AdminButtons_package = "<input class='nc_multi_check' type='checkbox' onclick='nc_package_obj.select(".$f_RowID.", ".$cc.");' >\r\n";
   
      if ($list_mode=='select' ) {
        $f_AdminButtons_buttons = "<a href='#' onclick='".$f_AdminButtons_select."'><img src='".$ADMIN_TEMPLATE."img/i_obj_select.gif' alt='".NETCAT_MODERATION_SELECT_RELATED."' title='".NETCAT_MODERATION_SELECT_RELATED."'></a>";
        $f_AdminButtons = "<div class='nc_idtab".($f_Checked ? "" : " nc_idtab_disabled")."'>";
        $f_AdminButtons.= "<div class='nc_idtab_handler' id='message".$classID."-".$f_RowID."_handler'></div>";
        $f_AdminButtons.= "<div class='nc_idtab_id'><div class='nc_idtab_ccid'>".NETCAT_MODERATION_ID.": <b>".$f_AdminButtons_id."</b></div></div>";
        $f_AdminButtons.= "<div class='nc_idtab_buttons'>".$f_AdminButtons_buttons."</div>";
        $f_AdminButtons.= "</div>";
        $f_AdminButtons.= "<div class='ncf_row nc_clear'></div>";
      }
      else {
        if ($system_env['AdminButtonsType']) {
            eval("\$f_AdminButtons = \"".$system_env['AdminButtons']."\";");
        }
        else {
          $f_AdminButtons_buttons = $f_AdminButtons_package;
          $f_AdminButtons_buttons.= "<a href='#' onclick=\"window.open('".$f_AdminButtons_copy."', 'nc_popup_test1', 'width=800,height=500,menubar=no,resizable=no,scrollbars=no,toolbar=no,resizable=yes'); return false;\"><img src='".$ADMIN_TEMPLATE."img/i_obj_copy.gif' alt='".NETCAT_MODERATION_COPY_OBJECT."' title='".NETCAT_MODERATION_COPY_OBJECT."'></a>";
          $f_AdminButtons_buttons.= "<a href='".$f_AdminButtons_check."'><img	src='".$ADMIN_TEMPLATE."img/i_obj_turn" . ($f_Checked ? 'on' : 'off' ) . ".gif' ".($f_Checked ? "alt='" . NETCAT_MODERATION_TURNTOOFF ."'  title='".NETCAT_MODERATION_TURNTOOFF."'"	: "alt='".NETCAT_MODERATION_TURNTOON."' title='".NETCAT_MODERATION_TURNTOON."'")."></a>";
          $f_AdminButtons_buttons .= "&nbsp;&nbsp;";
          $f_AdminButtons_buttons .= "<a href='".$f_AdminButtons_change."'><img src='".$ADMIN_TEMPLATE."img/i_obj_edit.gif' alt='".NETCAT_MODERATION_CHANGE."' title='".NETCAT_MODERATION_CHANGE."'></a>";
          $f_AdminButtons_buttons .= "&nbsp;&nbsp;";
          $f_AdminButtons_buttons.= "<a	href='".$f_AdminButtons_delete."'><img src='".$ADMIN_TEMPLATE."img/i_obj_delete.gif' alt='".NETCAT_MODERATION_DELETE."' title='".NETCAT_MODERATION_DELETE."'></a>";
          
          
          
          $f_AdminButtons = "<div class='nc_idtab".($f_Checked ? "" : " nc_idtab_disabled")."'>";
          // проверка прав
          if ( $modPerm || ($changePerm && $f_AdminButtons_user_add==$AUTH_USER_ID) ) {
            $f_AdminButtons.= "<div class='nc_idtab_handler' id='message".$classID."-".$f_RowID."_handler'></div>";
            $f_AdminButtons.= "<div class='nc_idtab_id'>";
            $f_AdminButtons.= "<div class='nc_idtab_messageid' 	title='".NETCAT_MODERATION_ID.": ".$f_AdminButtons_id."'>".$f_AdminButtons_id."</div>";
            $f_AdminButtons.= "<div class='nc_idtab_priority' 	title='".NETCAT_MODERATION_PRIORITY.": ".$f_AdminButtons_priority."'>".$f_AdminButtons_priority."</div>";
            $f_AdminButtons.= "<div class='nc_idtab_adduser' 		title='".NETCAT_MODERATION_ADDEDON.": ".$f_AdminInterface_user_add."'>".( $f_AdminInterface_user_add ? $f_AdminInterface_user_add : $f_AdminButtons_user_add )."</div>";
            $f_AdminButtons.= $f_AdminButtons_user_change ? "<div class='nc_idtab_edituser' title='".NETCAT_MODERATION_CHANGED.": ".$f_AdminInterface_user_change."'>".($f_AdminInterface_user_change ? $f_AdminInterface_user_change : $f_AdminButtons_user_change)."</div>" : "";
            $f_AdminButtons.= "</div>";
            $f_AdminButtons.= "<div class='nc_idtab_buttons'>".$f_AdminButtons_buttons."</div>";
            $f_AdminButtons.= "<div class='ncf_row nc_clear'></div>";
          }
          else { // прав нет
            $f_AdminButtons.= "<div class='nc_idtab_handler' id='message".$classID."-".$f_RowID."_handler'></div>";
            $f_AdminButtons.= "<div class='nc_idtab_id'>";
            $f_AdminButtons.= "<div class='nc_idtab_messageid error' title='".NETCAT_MODERATION_ERROR_NORIGHT."'>".NETCAT_MODERATION_ERROR_NORIGHT."</div>";
            $f_AdminButtons.= "</div>";
            $f_AdminButtons.= "<div class='ncf_row nc_clear'></div>";
          }
          $f_AdminButtons.= "</div>";
          $f_AdminButtons.= "<div class='ncf_row nc_clear'></div>";
        }
      }
      if ($user_table_mode) $f_AdminButtons = "";
    }
    else {
      $f_AdminButtons_id = "";
      $f_AdminButtons_priority = "";
      $f_AdminButtons_user_add = "";
      $f_AdminButtons_user_change = "";
      $f_AdminButtons_copy = "";
      $f_AdminButtons_change = "";
      $f_AdminButtons_delete = "";
      $f_AdminButtons_check = "";
      $f_AdminButtons_select = "";
      $f_AdminButtons = "";
      
      //$msgLink = ($f_Keyword && $f_Sub_Class_ID==$cc) ? $f_Keyword : $cc_keyword."_".$f_RowID;
      $msgLink = ($f_Keyword) ? $f_Keyword : $cc_keyword."_".$f_RowID;
      $dateLink = '';
      if ($date_field) eval("\$dateLink = \$f_".$date_field."_year.\"/\".\$f_".$date_field."_month.\"/\".\$f_".$date_field."_day.\"/\";");
      
      if ($catalogue == $current_catalogue['Catalogue_ID']) {
        $fullLink = $subLink.$msgLink.".html"; // полный вывод
        $fullRSSLink = $cc_env['AllowRSS'] ? $subLink.$msgLink.".rss" : ""; // rss
        $fullDateLink = $subLink.$dateLink.$msgLink.".html"; // полный вывод с датой
        $editLink  = $subLink."edit_".$msgLink.".html"; // ccылка для редактирования
        $deleteLink = $subLink."delete_".$msgLink.".html"; // удаления 
        $dropLink = $subLink."drop_".$msgLink.".html".($nc_core->token->is_use('drop') ? "?".$nc_core->token->get_url() : ""); // удаления без подтверждения
        $checkedLink = $subLink."checked_".$msgLink.".html";// включения\выключения
        $subscribeMessageLink = $subLink."subscribe_".$msgLink.".html"; // подписка на объект
      }
      else {
        $fullLink = $subHost.$subLink.$msgLink.".html"; ;
        $fullRSSLink = $cc_env['AllowRSS'] ? $subHost.$subLink.$msgLink.".html" : ""; ;
        $fullDateLink = $subHost.$subLink.$dateLink.$msgLink.".html";
        $editLink = $subHost."edit_".$msgLink.".html";
        $deleteLink = $subHost."delete_".$msgLink.".html";
        $dropLink = $subHost."drop_".$msgLink.".html";
        $checkedLink = $subHost."checked_".$msgLink.".html";
        $subscribeMessageLink = $subHost."subscribe_".$msgLink.".html"; // подписка на объект
      }
      
      // Если это превью данного компонента то, мы добавляем перенменную к ссылкам на полный просмотр объекта
      if ($classPreview == $cc_env["Class_ID"]) {
        $fullLink.= "?classPreview=".$classPreview;
        $fullDateLink.="?classPreview=".$classPreview;
      }
    }

    eval($cc_env['convert2txt']);
    eval("\$row = \"".nc_preg_replace('/\$result\b/', '$row', $cc_env["RecordTemplate"])."\";");

    // внутри админки: для того, чтобы объекты можно было перетаскивать...
    // ... сделаем "обертку" с ID, номером класса и ID родителя:
    if ($inside_admin) {
      $row_id_string = "id='message".$classID."-".$f_RowID."' messageParent='".$parent_message."' messageClass='".$classID."' messageSubclass='".$cc."' dragLabel='".htmlspecialchars($cc_env['Class_Name']." #".$f_RowID, ENT_QUOTES)."'";
      // попытаемся найти тэг, в который вложена строка...
      if ( nc_preg_match("@^\s*<(\w+).+</\\1>\s*$@s", $row, $regs) ) {
        $row = nc_preg_replace("@^(\s*<".$regs[1].")@s", "$1 ".$row_id_string, $row);
      } // если не удалось - добавим <div>
      else {
        $row = "<div ".$row_id_string.">".$row."</div>";
      }
    }

    $result.= ($no_cache_marks ? "<!-- nocache_object_".$f_RowNum." -->" : "").$row.($no_cache_marks ? "<!-- /nocache_object_".$f_RowNum." -->" : "");
  }

  // component suffix
  if ($cc_env['FormSuffix'] && !$ignore_suffix) {
    eval("\$result .= \"".$cc_env["FormSuffix"]."\";");
  }
  // добавить скрипт для D&D
  if ( $inside_admin && !$user_table_mode && $perm->isSubClassAdmin($cc) ) {
    // приоритет позволять менять только если отсортировано по умолчанию (Priority DESC)
    $change_priority = $cc_env['SortBy'] || $query_order ? 'false' : 'true';
    $result.= "<script type='text/javascript' language='Javascript'>";
    $result.= "if (typeof formAsyncSaveEnabled!='undefined') messageInitDrag(".nc_array_json( array($classID=>$row_ids)).", ".$change_priority.");";
    $result.= "</script>";
  }
  
  // cache section
  if ( nc_module_check_by_keyword("cache") && $cc_env['Cache_Access_ID']==1 && is_object($nc_cache_list) && !$user_table_mode ) {
    try {
      $bytes = $nc_cache_list->add($sub, $cc, $cache_key, $result, $cache_vars);
      // drop cache marks
      if ($no_cache_marks) $result = $nc_cache_list->nocacheClear($result);
      // debug info
      if ($bytes) {
        $cache_debug_info = "Writed, sub[".$sub."], cc[".$cc."], Access_ID[".$cc_env['Cache_Access_ID']."], Lifetime[".$cc_env['Cache_Lifetime']."], bytes[".$bytes."]";
        $nc_cache_list->debugMessage($cache_debug_info, __FILE__, __LINE__, "ok");
      }
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_list->errorMessage($e);
    }
  }

  return $result;
}


/*
 * DEPRECATED FUNCTION
 */
function s_list_class($sub, $cc, $query_string = "", $show_in_admin_mode = FALSE) {
  // new function
  return nc_objects_list($sub, $cc, $query_string, $show_in_admin_mode);
}





function showSearchForm ($fldName, $fldType, $fldDoSearch, $fldFmt) {
	global $systemTableID, $db, $srchPat;

	$j = 0;
	for($i=0; $i<count($fldName); $i++) {
		$fld_prefix = "<div>";
	  $fld_suffix = "</div>\n";
    $fldNameTempl = $fld_prefix."<b>".$fldName[$i]."</b>: ";

		if (!$fldDoSearch[$i]) continue;

		switch($fldType[$i]) {
			case 1:   // Char
				$result .= $fldNameTempl."<br><input type='text' name='srchPat[".$j."]' size='50' maxlength='255' value='".htmlspecialchars(stripcslashes($srchPat[$j]), ENT_QUOTES)."'>".$fld_suffix;
 				$j++;
 			break;
 			
			case 3:   // Text
				$result .= $fldNameTempl."<br><input type='text' name='srchPat[".$j."]' size='50' maxlength='255' value='".htmlspecialchars(stripcslashes($srchPat[$j]), ENT_QUOTES)."'>".$fld_suffix;
				$j++;
 				break;

 			case 6:   // File
				$result .= $fldNameTempl."<br><input type='text' name='srchPat[".$j."]' size='50' maxlength='255' value='".htmlspecialchars(stripcslashes($srchPat[$j]), ENT_QUOTES)."'>".$fld_suffix;
 				$j++;
				 break;

 			case 2:   // Int
				$result .= $fldNameTempl."&nbsp;&nbsp;".NETCAT_MODERATION_MOD_FROM." <input type='text' name='srchPat[".$j."]' size='10' maxlength='16' value='".($srchPat[$j] ? (int)$srchPat[$j] : "")."'>";
				$j++;
				$result .= NETCAT_MODERATION_MOD_DON."<input type='text' name='srchPat[".$j."]' size='10' maxlength='16' value='".($srchPat[$j] ? (int)$srchPat[$j] : "")."'>".$fld_suffix;
 				$j++;
 				break;

 			case 7:   // Float
				$result .= $fldNameTempl."&nbsp;&nbsp;".NETCAT_MODERATION_MOD_FROM." <input type='text' name='srchPat[".$j."]' size='10' maxlength='16' value='".($srchPat[$j] ? (float)$srchPat[$j] : "")."'>";
				$j++;
				$result .= NETCAT_MODERATION_MOD_DON."<input name='srchPat[".$j."]' type='text' size='10' maxlength='16' value='".($srchPat[$j] ? (float)$srchPat[$j] : "")."'>".$fld_suffix;
 				$j++;
 				break;

 			case 4:   // List
				if ($fldFmt[$i]) {
 					$result .= $fldNameTempl."<br><select name='srchPat[".$j."]' size='1'>";
 					$result .= "<option value=''>".NETCAT_MODERATION_MODA."</option>";

 					$SortType = $SortDirection = 0;
 					$res = $db->get_row("SELECT `Sort_Type`, `Sort_Direction` FROM `Classificator` WHERE `Table_Name` = '".$db->escape($fldFmt[$i])."'", ARRAY_N);
 					if ($db->num_rows != 0) {
    				$row = $res;
    				$SortType = $row[0];
    				$SortDirection = $row[1];
 					}

 					$s = "SELECT * FROM `Classificator_".$db->escape($fldFmt[$i])."` ORDER BY ";
 					switch ($SortType) {
    				case 1: $s .= "`".$db->escape($fldFmt[$i])."_Name`"; break;
    				case 2: $s .= "`".$db->escape($fldFmt[$i])."_Priority`"; break;
    				default: $s .= "`".$db->escape($fldFmt[$i])."_ID`";
 					}

 					if ($SortDirection == 1) $s .= " DESC";

 					$selected = (int)$srchPat[$j];
     			$lstRes = $db->get_results($s, ARRAY_N);
     			foreach  ($lstRes as $q) {
        		list($lstID, $lstName) = $q;
        		$result .= "<option value='".$lstID."'".($selected == $lstID ? "selected" : "").">".$lstName."</option>";
 					}
 					$result .= '</select>'.$fld_suffix;
 				}
     		$j++;
     		break;

     	case 5: // Bool
 				$result .= $fldNameTempl;
 				$result .= "&nbsp;&nbsp;<input type='radio' name='srchPat[".$j."]' id='t".$j."_1' value='' style='vertical-align:middle'".(!$srchPat[$j] ? " checked" : "")."><label for='t".$j."_1'>".NETCAT_MODERATION_MOD_NOANSWER.'</label> ';
 				$result .= "&nbsp;&nbsp;<input type='radio' name='srchPat[".$j."]' id='t".$j."_2' value='1' style='vertical-align:middle'".($srchPat[$j]=='1' ? " checked" : "")."><label for='t".$j."_2'>".CONTROL_CONTENT_SUBDIVISION_FUNCS_OBJ_YES.'</label> ';
 				$result .= "&nbsp;&nbsp;<input type='radio' name='srchPat[".$j."]' id='t".$j."_3' value='0' style='vertical-align:middle'".($srchPat[$j]=='0' ? " checked" : "")."><label for='t".$j."_3'>".CONTROL_CONTENT_SUBDIVISION_FUNCS_OBJ_NO.'</label>';
 				$result .= $fld_suffix;
 				$j++;
				 break;

			 case 8:   // DateTime
 				$result .= $fldNameTempl."&nbsp;&nbsp;".NETCAT_MODERATION_MOD_FROM;
 				if ($fldFmt[$i] != 'event_time') {
 					$result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>."; $j++;
 					$result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>."; $j++;
 					$result .= "<input type='text' name='srchPat[".$j."]' size='4' maxlength='4' value='".($srchPat[$j] ? sprintf("%04d",$srchPat[$j]) : "")."'> "; $j++;
 				} else $j += 3;
				if ($fldFmt[$i] != 'event_date') {
				 $result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>:"; $j++;
				 $result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>:"; $j++;
				 $result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'> "; $j++;
				 } else $j += 3;

     	$result .= NETCAT_MODERATION_MOD_DON;
     	if ($fldFmt[$i] != 'event_time') {
 				$result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>."; $j++;
 				$result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>."; $j++;
 				$result .= "<input type='text' name='srchPat[".$j."]' size='4' maxlength='4' value='".($srchPat[$j] ? sprintf("%04d",$srchPat[$j]) : "")."'> "; $j++;
 			} else $j += 3;
 			if ($fldFmt[$i] != 'event_date') {
 				$result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>:"; $j++;
 				$result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'>:"; $j++;
 				$result .= "<input type='text' name='srchPat[".$j."]' size='2' maxlength='2' value='".($srchPat[$j] ? sprintf("%02d",$srchPat[$j]) : "")."'> "; $j++;
 			} else $j += 3;
				$result .= $fld_suffix;
     		break;
          
     	case 10:   //Multi List
     	  if ($fldFmt[$i]) { 
          list( $clft_name, $type_element, $type_size ) = explode(":", $fldFmt[$i]); 

          if ( !$type_element )  $type_element = "select"; 
          if (!$type_size )   $type_size = 3; 
    
          $fldFmt[$i] = $clft_name;
          
          $SortType = $SortDirection = 0;
          $res = $db->get_row("SELECT `Sort_Type`, `Sort_Direction` FROM `Classificator` WHERE `Table_Name` = '".$db->escape($fldFmt[$i])."'", ARRAY_N);
          if ($db->num_rows != 0) {
            $row = $res;
            $SortType = $row[0];
            $SortDirection = $row[1];
          }

 					$s = "SELECT * FROM Classificator_".$fldFmt[$i]." ORDER BY ";
 					switch ($SortType) {
    				case 1: $s .= $fldFmt[$i]."_Name"; break;
    				case 2: $s .= $fldFmt[$i]."_Priority"; break;
    				default: $s .= $fldFmt[$i]."_ID";
 					}

 					if ($SortDirection == 1) $s .= " DESC";

 					$selected = (int)$srchPat[$j];
     			$lstRes = $db->get_results($s, ARRAY_N);
     			
 					$result .= $fldNameTempl."<br>";
 					
 					if ($type_element == 'select') {
	 					$result .= "<select name='srchPat[".$j."][]' size='".$type_size."' multiple>";
	 					$result .= "<option value=''>".NETCAT_MODERATION_MODA."</option>";
 					}
		
     			
          foreach  ($lstRes as $q) {
            list($lstID, $lstName) = $q;
            $temp_str = '';
            if ( $lstID == $selected) $temp_str = ($type_element == "select" ) ? " selected" : " checked";
      
            if ( $type_element == 'select' ) {  #TODO сделать возможность передавать селектед в виде массива
              $result .= "<option value='".$lstID."' ".$temp_str.">".$lstName."</option>";
            }
            else {
              $result .="<input type='checkbox' value='".$lstID."' name='srchPat[".$j."][]' ".$temp_str."> ".$lstName."<br>\r\n";
            }
          }
 					
 					if ($type_element == 'select') $result .= '</select><br>';//.$fld_suffix;
 					
 					$j++;
 					$result .= "<input type='hidden' name='srchPat[".$j."]' value='0'>\n";
 					$result .= $fld_suffix;
 					
 				}
     		$j++;
     		break;

		}
  	$result .= "<br>\n";
 }
 
	if (!$j) return false;
 	
 	return $result;
}


function getSearchParams ($field_name, $field_type, $field_search, $srchPat) {
  global $db;
  
  // return if search params not setted
  if ( empty($srchPat) ) return array("query" => "", "link" => "");
  $search_param = array();
  for($i=0,$j=0; $i<count($field_name); $i++) {
    if ($field_search[$i]) {
      switch($field_type[$i]) {
        case 1:   // Char
          if($srchPat[$j] == "") break;
          $srch_str = $db->escape($srchPat[$j]);
          $fullSearchStr .= " AND a.".$field_name[$i]." LIKE '%".urldecode($srch_str)."%'";
          $search_param[] = "srchPat[".$j."]=".urldecode($srchPat[$j]);
          break;
        case 2:   // Int
          if($srchPat[$j] != "") {
             $srchPat[$j] += 0;
             $fullSearchStr .= " AND a.".$field_name[$i].">=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
          $j++;
          if($srchPat[$j] != "") {
             $srchPat[$j] += 0;
             $fullSearchStr .= " AND a.".$field_name[$i]."<=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
          break;
        case 3:   // Text
          if($srchPat[$j] == "") break;
          $srch_str = $db->escape($srchPat[$j]);
          $fullSearchStr .= " AND a.".$field_name[$i]." LIKE '%".urldecode($srch_str)."%'";
          $search_param[] = "srchPat[".$j."]=".urldecode($srchPat[$j]);
          break;
        case 4: // List
          if($srchPat[$j] == "") break;
          $srchPat[$j] += 0;
          $fullSearchStr .= " AND a.".$field_name[$i]."=".$srchPat[$j];
          $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          break;
        case 5:   // Boolean
          if($srchPat[$j] == "") break;
          $srchPat[$j] += 0;
          $fullSearchStr .= " AND a.".$field_name[$i]."=".$srchPat[$j];
          $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          break;
        case 6:   // File
          if($srchPat[$j] == "") break;
          $srch_str = $db->escape($srchPat[$j]);
          $fullSearchStr .= " AND SUBSTRING_INDEX(a.".$field_name[$i].",':',1) LIKE '%".urldecode($srch_str)."%'";
          $search_param[] = "srchPat[".$j."]=".urldecode($srchPat[$j]);
          break;
        case 7:   // Float
          if($srchPat[$j] != "") {
             $srchPat[$j] = floatval($srchPat[$j]);
             $fullSearchStr .= " AND a.".$field_name[$i].">=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
          $j++;
          if($srchPat[$j] != "") {
             $srchPat[$j] = floatval($srchPat[$j]);
             $fullSearchStr .= " AND a.".$field_name[$i]."<=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
        break;
        case 8:   // DateTime
          $date_from['d'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['m'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['Y'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%04d", $srchPat[$j]) : false); $j++;
          $date_from['H'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['i'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['s'] =($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['d'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['m'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['Y'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%04d", $srchPat[$j]) : false); $j++;
          $date_to['H'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['i'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['s'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false);

          $date_format_from = ($date_from['Y'] ? '%Y' : '').($date_from['m'] ? '%m' : '').($date_from['d'] ? '%d' : '').($date_from['H'] ? '%H' : '').($date_from['i'] ? '%i' : '').($date_from['s'] ? '%s' : '');
          $date_format_to = ($date_to['Y'] ? '%Y' : '').($date_to['m'] ? '%m' : '').($date_to['d'] ? '%d' : '').($date_to['H'] ? '%H' : '').($date_to['i'] ? '%i' : '').($date_to['s'] ? '%s' : '');

          if($date_format_from) $fullSearchStr .= " AND DATE_FORMAT(a.".$field_name[$i].",'".$date_format_from."')>=".$date_from['Y'].$date_from['m'].$date_from['d'].$date_from['H'].$date_from['i'].$date_from['s'];
          if($date_format_to) $fullSearchStr .= " AND DATE_FORMAT(a.".$field_name[$i].",'".$date_format_to."')<=".$date_to['Y'].$date_to['m'].$date_to['d'].$date_to['H'].$date_to['i'].$date_to['s'];

          break;
          
          case 10: // MultiList
            if($srchPat[$j] == ""){ $j++; break; }

            $id = array(); // массив с id искомых элементов
            
            if (is_array($srchPat[$j])) {
              foreach ( (array)$srchPat[$j] as $v ) {
                $id[] = intval($v);
              }
            }
            else {
              $temp_id = explode('-', $srchPat[$j]);
              foreach ( (array)$temp_id as  $v ) {
                $id[] = intval($v);
              }
            }
            $j++; //второй параметр - это тип посика
            
            if ( empty($id) ) break;
            
            $fullSearchStr .= " AND (";
            switch ($srchPat[$j]) {
              case 1: //Полное совпадение
                $fullSearchStr .= "a.".$field_name[$i]." LIKE CONCAT(',' ,  '".join(',', $id)."', ',') ";
                break;
              
              case 2: //Хотя бы один. Выбор между LIKE и REGEXP выпал в сторону первого
                foreach ( $id as $v ) $fullSearchStr .= "a.".$field_name[$i]." LIKE CONCAT('%,', '".$v."', ',%') OR ";
                $fullSearchStr .= "0 "; //чтобы "закрыть" последний OR         
                break;
             case 0: // как минимум выбранные - частичное совпадение - по умолчанию
              default:
                $srchPat[$j] = 0; 
                $fullSearchStr .= "a.".$field_name[$i]."  REGEXP  \"((,[0-9]+)*)";
                $prev_v =  -1;
                foreach ( $id as $v ) {
                  $fullSearchStr .= "(,".$v.",)([0-9]*)((,[0-9]+)*)";
                  $prev_v = $v;
                }
                $fullSearchStr .= '"';
                break;     
            }
            $fullSearchStr .= ")";
            
            $search_param[] = "srchPat[".($j-1)."]=".join('-', $id);
            $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          break;
      }
      $j++;
    }
  }
  
  if ( !empty($search_param)) $search_params['link'] = join('&amp;', $search_param);
  $search_params['query'] = $fullSearchStr;

  return $search_params;
}

?>
